#!/bin/sh

mkdir all_demos;
mv aa_demo/aa_demo								all_demos/aa_demo;
mv alpha_mask/alpha_mask						all_demos/alpha_mask;
mv alpha_mask2/alpha_mask2						all_demos/alpha_mask2;
mv alpha_mask3/alpha_mask3						all_demos/alpha_mask3;
mv circles/circles								all_demos/circles;
mv component_rendering/component_rendering		all_demos/component_rendering;
mv conv_contour/conv_contour					all_demos/conv_contour;
mv conv_dash_marker/conv_dash_marker			all_demos/conv_dash_marker;
mv conv_stroke/conv_stroke						all_demos/conv_stroke;
mv distortions/distortions						all_demos/distortions;
mv gamma_correction/gamma_correction			all_demos/gamma_correction;
mv gamma_ctrl/gamma_ctrl						all_demos/gamma_ctrl;
mv gouraud/gouraud								all_demos/gouraud;
mv gpc_test/gpc_test							all_demos/gpc_test;
mv gradients/gradients							all_demos/gradients;
mv graph_test/graph_test						all_demos/graph_test;
mv idea/idea									all_demos/idea;
mv image_alpha/image_alpha						all_demos/image_alpha;
mv image_filters/image_filters					all_demos/image_filters;
mv image_fltr_graph/image_fltr_graph			all_demos/image_fltr_graph;
mv image_perspective/image_perspective			all_demos/image_perspective;
mv image_transforms/image_transforms			all_demos/image_transforms;
mv image1/image1								all_demos/image1;
mv lion/lion									all_demos/lion;
mv lion_lens/lion_lens							all_demos/lion_lens;
mv lion_outline/lion_outline					all_demos/lion_outline;
mv mol_view/mol_view							all_demos/mol_view;
mv multi_clip/multi_clip						all_demos/multi_clip;
mv pattern_fill/pattern_fill					all_demos/pattern_fill;
mv perspective/perspective						all_demos/perspective;
mv polymorphic_renderer/polymorphic_renderer	all_demos/polymorphic_renderer;
mv rasterizers/rasterizers						all_demos/rasterizers;
mv rasterizers2/rasterizers2					all_demos/rasterizers2;
mv rounded_rect/rounded_rect					all_demos/rounded_rect;
mv simple_blur/simple_blur						all_demos/simple_blur;
mv trans_polar/trans_polar						all_demos/trans_polar;
